@@ -10,6 +10,7 @@ import time
 import jwt
 from typing import Any, Dict, Optional
 from django.contrib.auth import get_user_model
+from django.conf import settings
 from django_bolt.types import Request
 
 
@@ -67,7 +68,6 @@ def create_jwt_for_user(
     """
     # Use Django SECRET_KEY if no secret provided
     if secret is None:
-        from django.conf import settings
         secret = settings.SECRET_KEY
 
     # Build standard claims
@@ -187,7 +187,7 @@ def get_auth_context(request: Request) -> Dict[str, Any]:
         Authentication context dictionary containing:
         - user_id: User identifier
         - is_staff: Staff status boolean
-        - is_admin: Admin/superuser status boolean
+        - is_superuser: Superuser status boolean
         - auth_backend: Authentication backend used (jwt, api_key, etc.)
         - permissions: List of permissions (if available)
         - auth_claims: JWT claims dict (if JWT auth was used)
@@ -198,7 +198,7 @@ def get_auth_context(request: Request) -> Dict[str, Any]:
         async def admin_stats(request: dict):
             auth_ctx = get_auth_context(request)
 
-            if not auth_ctx.get("is_admin"):
+            if not auth_ctx.get("is_superuser"):
                 return {"error": "Admin access required"}
 
             # Use user_id for audit logging
