@@ -1,25 +1,35 @@
-from typing import Optional, List, Annotated
-from django_bolt.types import Request
-import msgspec
 import asyncio
-import time
 import json
-from django_bolt import BoltAPI, JSON, OpenAPIConfig, SwaggerRenderPlugin, RedocRenderPlugin
-from django_bolt.views import APIView, ViewSet
-from django_bolt.param_functions import Header, Cookie, Form, File
-from django_bolt.responses import PlainText, HTML, Redirect, FileResponse, StreamingResponse
+import os
+import time
+from typing import Annotated, List, Optional, Protocol
+
+import msgspec
+from msgspec import Meta, ValidationError
+
+from django.contrib.auth import aauthenticate, get_user_model
+
+from django_bolt import BoltAPI, CompressionConfig, JSON, OpenAPIConfig, RedocRenderPlugin, SwaggerRenderPlugin, action
+from django_bolt.auth import JWTAuthentication, IsAuthenticated, create_jwt_for_user, get_current_user
 from django_bolt.exceptions import (
+    BadRequest,
     HTTPException,
     NotFound,
-    BadRequest,
+    RequestValidationError,
     Unauthorized,
     UnprocessableEntity,
-    RequestValidationError,
 )
-from django_bolt.health import register_health_checks, add_health_check
-from django_bolt.middleware import no_compress, cors
-from django_bolt import CompressionConfig
+from django_bolt.health import add_health_check, register_health_checks
+from django_bolt.middleware import cors, no_compress
+from django_bolt.param_functions import Cookie, Depends, File, Form, Header
+from django_bolt.responses import FileResponse, HTML, PlainText, Redirect, StreamingResponse
+from django_bolt.serializers import Serializer, field_validator, model_validator
+from django_bolt.types import Request
+from django_bolt.views import APIView, ViewSet
 
+import test_data
+from users.api import UserMini
+from users.models import User
 # OpenAPI is enabled by default at /docs with Swagger UI
 # You can customize it by passing openapi_config:
 #
@@ -66,14 +76,201 @@ class Item(msgspec.Struct):
     is_offer: Optional[bool] = None
 
 
-import test_data
-
 @api.get("/health")
 async def health():
     """Health check endpoint."""
     return {"status": "healthy", "timestamp": time.time()}
 
 
+class CustomRequest(Request, Protocol):
+    """Extended Request type with custom properties"""
+
+    # Inherited from Request:
+    # - method, path, body, context, user
+    # - get(), __getitem__()
+
+    # If you add custom request properties via middleware:
+    tenant_id: Optional[str]
+    request_id: str
+
+
+# ==== Authentication Examples - JWT Auth with request.user ====
+@api.get(
+    "/auth/me",
+    auth=[JWTAuthentication()],
+    guards=[IsAuthenticated()],
+    tags=["auth"],
+    summary="Get current authenticated user"
+)
+async def get_me(request: CustomRequest):
+    """
+    Returns the authenticated user's information using request.user.
+
+    Requires a valid JWT token in the Authorization header:
+    Authorization: Bearer <jwt_token>
+
+    The request.user property is automatically populated by the authentication
+    system and contains the Django User instance for the authenticated user.
+    """
+    # Debug logging
+    context = request.get("context", {})
+    user_id = context.get("user_id")
+    auth_backend = context.get("auth_backend")
+    print(f"DEBUG: user_id={user_id}, auth_backend={auth_backend}", flush=True)
+
+    user = request.user
+    print(f"DEBUG: request.user={user}, type={type(user)}", flush=True)
+
+    if not user:
+        return {"error": "User not authenticated"}
+
+    return {
+        "id": user.id,
+        "username": user.username,
+        "email": user.email,
+        "is_staff": user.is_staff,
+        "is_superuser": user.is_superuser,
+        "is_active": user.is_active,
+        "first_name": user.first_name,
+        "last_name": user.last_name,
+    }
+
+
+@api.get(
+    "/auth/me-dependency",
+    auth=[JWTAuthentication()],
+    guards=[IsAuthenticated()],
+    tags=["auth"],
+    summary="Get current user via dependency injection"
+)
+async def get_me_dependency(user=Depends(get_current_user)):
+    """
+    Alternative endpoint that uses dependency injection to get the current user.
+
+    This demonstrates the `get_current_user` dependency which is useful when
+    you want to ensure the user is loaded early and available for other operations.
+
+    Requires a valid JWT token in the Authorization header:
+    Authorization: Bearer <jwt_token>
+    """
+    if not user:
+        return {"error": "User not authenticated"}
+
+    return {
+        "id": user.id,
+        "username": user.username,
+        "email": user.email,
+        "is_staff": user.is_staff,
+        "is_superuser": user.is_superuser,
+        "is_active": user.is_active,
+        "first_name": user.first_name,
+        "last_name": user.last_name,
+    }
+
+
+@api.get(
+    "/auth/context",
+    auth=[JWTAuthentication()],
+    guards=[IsAuthenticated()],
+    tags=["auth"],
+    summary="Get authentication context",
+    preload_user=False
+)
+async def get_auth_context(request: Request):
+    """
+    Returns the raw authentication context (header-based access).
+
+    This shows what's available in the request.context dictionary,
+    including auth backend info and user claims from the JWT.
+
+    Requires a valid JWT token in the Authorization header:
+    Authorization: Bearer <jwt_token>
+    """
+    context = request.context
+
+    return {
+        "user_id": context.get("user_id"),
+        "is_staff": context.get("is_staff"),
+        "is_superuser": context.get("is_superuser"),
+        "auth_backend": context.get("auth_backend"),
+        "permissions": context.get("permissions", []),
+        "auth_claims": context.get("auth_claims", {}),
+    }
+
+
+class TokenRequest(msgspec.Struct):
+    """Request body for token generation."""
+    username: str
+    password: str
+
+
+@api.post(
+    "/auth/token",
+    tags=["auth"],
+    summary="Generate access token"
+)
+async def generate_token(token_req: TokenRequest):
+    """
+    Generate a JWT access token for a user.
+
+    Accepts username and password, validates them, and returns a JWT token.
+
+    Request body:
+    ```json
+    {
+        "username": "john",
+        "password": "password123"
+    }
+    ```
+
+    Response on success:
+    ```json
+    {
+        "access_token": "eyJ0eXAiOiJKV1QiLCJhbGc...",
+        "token_type": "bearer",
+        "expires_in": 3600,
+        "user": {
+            "id": 1,
+            "username": "john",
+            "email": "john@example.com",
+            "is_staff": false,
+            "is_superuser": false
+        }
+    }
+    ```
+
+    Response on failure (invalid credentials):
+    ```json
+    {
+        "error": "Invalid username or password"
+    }
+    ```
+    """
+    User = get_user_model()
+
+    # Authenticate the user
+    user = await aauthenticate(username=token_req.username, password=token_req.password)
+
+    if user is None:
+        raise Unauthorized(detail="Invalid username or password")
+
+    # Generate JWT token
+    access_token = create_jwt_for_user(user, expires_in=3600)
+
+    return {
+        "access_token": access_token,
+        "token_type": "bearer",
+        "expires_in": 3600,
+        "user": {
+            "id": user.id,
+            "username": user.username,
+            "email": user.email,
+            "is_staff": user.is_staff,
+            "is_superuser": user.is_superuser,
+        },
+    }
+
+
 @api.get("/", tags=["root"], summary="summary", description="description")
 @cors()  # Uses global CORS_ALLOWED_ORIGINS from Django settings
 async def read_root():
@@ -82,15 +279,76 @@ async def read_root():
     """
     return {"message": "Hello World"}
 
+@api.get("/sync", tags=["root"], summary="summary", description="description")
+@cors()  # Uses global CORS_ALLOWED_ORIGINS from Django settings
+def read_root():
+    """
+    Endpoint that returns a simple "Hello World" dictionary.
+    """
+    return {"message": "Hello World"}
+
 
 @api.get("/10k-json")
 async def read_10k():
     """
     Endpoint that returns 10k JSON objects.
-    
+
     """
     return test_data.JSON_10K
 
+
+@api.get("/sync-10k-json")
+def read_10k_sync():
+    """
+    Sync version: Endpoint that returns 10k JSON objects.
+
+    """
+    return test_data.JSON_10K
+
+
+class UserMiniSerializer(Serializer):
+    id: int
+    username:str
+
+
+@api.get("/sync-users", response_model=list[UserMini])
+def read_10k_sync():
+    """
+    Sync version: Endpoint that returns 10k JSON objects.
+
+    """
+    users = User.objects.all()[0:100]
+
+    return users
+
+
+# @api.get("/sync-users")
+# def read_10k_sync() -> list[UserMini]:
+#     """
+#     Sync version: Endpoint that returns 10k JSON objects.
+
+#     """
+#     users = User.objects.all()[0:100]
+
+#     return users
+
+
+@api.get("/async-users")
+async def read_10k_sync() -> list[UserMini]:
+    """
+    Sync version: Endpoint that returns 10k JSON objects.
+
+    """
+    users = User.objects.all()[0:100]
+
+    return users
+    # users = list(users)
+    # return users
+    
+    return User.objects.all()[:100]
+
+
+
 @api.get("/items/{item_id}")
 async def read_item(item_id: int, q: Optional[str] = None):
     return {"item_id": item_id, "q": q}
@@ -196,7 +454,6 @@ async def handle_mixed(
 
 
 # ==== File serving endpoint for benchmarks ====
-import os
 THIS_FILE = os.path.abspath(__file__)
 
 
@@ -209,13 +466,14 @@ async def file_static():
     return FileResponse("/path/to/nonexistent/file.txt", filename="asdfasd.py")
 
 # ==== Streaming endpoints for benchmarks ====
+#TODO: Add proper api for streaming files 
 @api.get("/stream")
 @no_compress
 async def stream_plain():
     def gen():
         for i in range(100):
             yield "x"
-    return StreamingResponse(gen, media_type="text/plain")
+    return StreamingResponse(gen(), media_type="text/plain")
 
 
 @api.get("/collected")
@@ -226,10 +484,22 @@ async def collected_plain():
 @api.get("/sse")
 @no_compress
 async def sse():
+    async def gen():
+        while True:
+            await asyncio.sleep(1)
+            yield f"data: {time.time()}\n\n"
+    return StreamingResponse(gen(), media_type="text/event-stream")
+
+
+@api.get("/sync-sse")
+@no_compress
+def sse_sync():
+    """Sync version: Server-Sent Events."""
     def gen():
-        for i in range(3):
-            yield f"data: {i}\n\n"
-    return StreamingResponse(gen, media_type="text/event-stream")
+        while True:
+            time.sleep(1)
+            yield f"data: {time.time()}\n\n"
+    return StreamingResponse(gen(), media_type="text/event-stream")
 
 
 # ==== OpenAI-style Chat Completions (streaming/non-streaming) ====
@@ -264,236 +534,56 @@ class ChatCompletionChunk(msgspec.Struct):
     object: str = "chat.completion.chunk"
 
 
-@api.post("/v1/chat/completions")
+@api.get("/v1/chat/completions")
 @no_compress
-async def openai_chat_completions(payload: ChatCompletionRequest):
+async def openai_chat_completions():
+    """Ultra-optimized chat completions endpoint that streams 100 chunks using msgspec"""
     created = int(time.time())
-    model = payload.model or "gpt-4o-mini"
+    model = "gpt-4o-mini"
     chat_id = "chatcmpl-bolt-bench"
 
-    if payload.stream:
-        def gen():
-            delay = max(0, payload.delay_ms or 0) / 1000.0
-            for i in range(max(1, payload.n_chunks)):
-                data = {
-                    "id": chat_id,
-                    "object": "chat.completion.chunk",
-                    "created": created,
-                    "model": model,
-                    "choices": [
-                        {"index": 0, "delta": {"content": payload.token}, "finish_reason": None}
-                    ],
-                }
-                yield f"data: {json.dumps(data, separators=(',', ':'))}\n\n"
-                if delay > 0:
-                    time.sleep(delay)
-            final = {
-                "id": chat_id,
-                "object": "chat.completion.chunk",
-                "created": created,
-                "model": model,
-                "choices": [
-                    {"index": 0, "delta": {}, "finish_reason": "stop"}
-                ],
-            }
-            yield f"data: {json.dumps(final, separators=(',', ':'))}\n\n"
-            yield "data: [DONE]\n\n"
-        return StreamingResponse(gen, media_type="text/event-stream")
+    # Pre-create reusable msgspec structs (minimal object creation)
+    stop_delta = ChatCompletionChunkDelta()
 
-    text = (payload.token * max(1, payload.n_chunks)).strip()
-    response = {
-        "id": chat_id,
-        "object": "chat.completion",
-        "created": created,
-        "model": model,
-        "choices": [
-            {"index": 0, "message": {"role": "assistant", "content": text}, "finish_reason": "stop"}
-        ],
-    }
-    return response
-
-
-@api.get("/sse-async")
-@no_compress
-async def sse_async():
     async def agen():
-        for i in range(3):
-            yield f"data: {i}\n\n"
-    return StreamingResponse(agen(), media_type="text/event-stream")
-
-@api.get("/sse-async-sleep")
-async def sse_async_sleep():
-    async def agen():
-        for i in range(3):
-            yield f"data: {i}\n\n"
-            await asyncio.sleep(0)
-    return StreamingResponse(agen(), media_type="text/event-stream")
-
-@api.get("/sse-async-batch")
-@no_compress
-async def sse_async_batch():
-    """Optimized async endpoint that yields all data at once to reduce overhead"""
-    async def agen():
-        # Batch all data into single yield to minimize GIL crossings
-        all_data = "".join(f"data: {i}\n\n" for i in range(3))
-        yield all_data
-    return StreamingResponse(agen(), media_type="text/event-stream")
-
-
-@api.post("/v1/chat/completions-async")
-@no_compress
-async def openai_chat_completions_async(payload: ChatCompletionRequest):
-    created = int(time.time())
-    model = payload.model or "gpt-4o-mini"
-    chat_id = "chatcmpl-bolt-bench-async"
-
-    if payload.stream:
-        async def agen():
-            import os
-            debug_timing = os.environ.get("DJANGO_BOLT_DEBUG_TIMING")
-            if debug_timing:
-                gen_start = time.time()
-                chunk_times = []
-            
-            delay = max(0, payload.delay_ms or 0) / 1000.0
-            for i in range(max(1, payload.n_chunks)):
-                if debug_timing:
-                    chunk_start = time.time()
-                
-                # ULTRA-OPTIMIZATION: Use msgspec structs for 5-10x faster JSON serialization
-                chunk = ChatCompletionChunk(
-                    id=chat_id,
-                    created=created,
-                    model=model,
-                    choices=[ChatCompletionChunkChoice(
-                        index=0,
-                        delta=ChatCompletionChunkDelta(content=payload.token),
-                        finish_reason=None
-                    )]
-                )
-                # msgspec.json.encode is 5-10x faster than json.dumps + much faster than dict creation
-                chunk_json = msgspec.json.encode(chunk)
-                
-                if debug_timing:
-                    serialize_time = time.time() - chunk_start
-                    yield_start = time.time()
-                
-                yield b"data: " + chunk_json + b"\n\n"
-                
-                if debug_timing:
-                    total_chunk_time = time.time() - chunk_start
-                    chunk_times.append((serialize_time * 1000, total_chunk_time * 1000))
-                    if i == 0:  # Log first chunk timing
-                        print(f"[PY-TIMING] First chunk: serialize={serialize_time*1000:.3f}ms, total={total_chunk_time*1000:.3f}ms")
-                
-                if delay > 0:
-                    await asyncio.sleep(delay)
-                    
-            # Final chunk with msgspec optimization
-            final_chunk = ChatCompletionChunk(
-                id=chat_id,
-                created=created,
-                model=model,
-                choices=[ChatCompletionChunkChoice(
-                    index=0,
-                    delta=ChatCompletionChunkDelta(),
-                    finish_reason="stop"
-                )]
-            )
-            final_json = msgspec.json.encode(final_chunk)
-            yield b"data: " + final_json + b"\n\n"
-            yield b"data: [DONE]\n\n"
-            
-            if debug_timing:
-                total_gen_time = (time.time() - gen_start) * 1000
-                avg_serialize = sum(t[0] for t in chunk_times) / len(chunk_times) if chunk_times else 0
-                avg_total = sum(t[1] for t in chunk_times) / len(chunk_times) if chunk_times else 0
-                print(f"[PY-TIMING] Generator complete: total={total_gen_time:.3f}ms, avg_serialize={avg_serialize:.3f}ms, avg_chunk={avg_total:.3f}ms")
-                
-        return StreamingResponse(agen(), media_type="text/event-stream")
-
-    # Non-streaming identical to sync path
-    text = (payload.token * max(1, payload.n_chunks)).strip()
-    response = {
-        "id": chat_id,
-        "object": "chat.completion",
-        "created": created,
-        "model": model,
-        "choices": [
-            {"index": 0, "message": {"role": "assistant", "content": text}, "finish_reason": "stop"}
-        ],
-    }
-    return response
-
-
-@api.post("/v1/chat/completions-ultra")
-@no_compress
-async def openai_chat_completions_ultra_optimized(payload: ChatCompletionRequest):
-    """Ultra-optimized version with msgspec structs and minimal allocations."""
-    created = int(time.time())
-    model = payload.model or "gpt-4o-mini"
-    chat_id = "chatcmpl-bolt-ultra"
-
-    if payload.stream:
-        # Pre-create reusable msgspec structs (minimal object creation)
-        token_delta = ChatCompletionChunkDelta(content=payload.token)
-        stop_delta = ChatCompletionChunkDelta()
-        
-        async def ultra_agen():
-            delay = max(0, payload.delay_ms or 0) / 1000.0
-            
-            # Ultra-optimized: reuse structs and minimize allocations
-            for _ in range(max(1, payload.n_chunks)):
-                # Reuse pre-created delta struct
-                choice = ChatCompletionChunkChoice(
-                    index=0,
-                    delta=token_delta,
-                    finish_reason=None
-                )
-                chunk = ChatCompletionChunk(
-                    id=chat_id,
-                    created=created,
-                    model=model,
-                    choices=[choice]
-                )
-                
-                # msgspec.json.encode directly to bytes - fastest possible path
-                chunk_bytes = msgspec.json.encode(chunk)
-                yield b"data: " + chunk_bytes + b"\n\n"
-                
-                if delay > 0:
-                    await asyncio.sleep(delay)
-            
-            # Final chunk with stop reason
-            final_choice = ChatCompletionChunkChoice(
+        # Ultra-optimized: reuse structs and minimize allocations
+        for i in range(50):
+            await asyncio.sleep(0.2)
+            # Reuse pre-created delta struct
+            choice = ChatCompletionChunkChoice(
                 index=0,
-                delta=stop_delta,
-                finish_reason="stop"
+                delta=ChatCompletionChunkDelta(content=f"hello - {i}"),
+                finish_reason=None
             )
-            final_chunk = ChatCompletionChunk(
+            chunk = ChatCompletionChunk(
                 id=chat_id,
                 created=created,
                 model=model,
-                choices=[final_choice]
+                choices=[choice]
             )
-            final_bytes = msgspec.json.encode(final_chunk)
-            yield b"data: " + final_bytes + b"\n\n"
-            yield b"data: [DONE]\n\n"
-            
-        return StreamingResponse(ultra_agen(), media_type="text/event-stream")
 
-    # Non-streaming path unchanged
-    text = (payload.token * max(1, payload.n_chunks)).strip()
-    response = {
-        "id": chat_id,
-        "object": "chat.completion",
-        "created": created,
-        "model": model,
-        "choices": [
-            {"index": 0, "message": {"role": "assistant", "content": text}, "finish_reason": "stop"}
-        ],
-    }
-    return response
+            # msgspec.json.encode directly to bytes - fastest possible path
+            chunk_bytes = msgspec.json.encode(chunk)
+            yield b"data: " + chunk_bytes + b"\n\n"
+
+        # Final chunk with stop reason
+        final_choice = ChatCompletionChunkChoice(
+            index=0,
+            delta=stop_delta,
+            finish_reason="stop"
+        )
+        final_chunk = ChatCompletionChunk(
+            id=chat_id,
+            created=created,
+            model=model,
+            choices=[final_choice]
+        )
+        final_bytes = msgspec.json.encode(final_chunk)
+        yield b"data: " + final_bytes + b"\n\n"
+        yield b"data: [DONE]\n\n"
+
+    return StreamingResponse(agen(), media_type="text/event-stream")
+
 
 # ==== Error Handling & Logging Examples ====
 
@@ -661,6 +751,69 @@ async def compression_test():
     return large_data
 
 
+# ============================================================================
+# Serializer Benchmark Endpoints
+# ============================================================================
+
+class BenchAuthorRaw(msgspec.Struct):
+    """Raw msgspec for baseline comparison."""
+    id: int
+    name: Annotated[str, Meta(min_length=2)]
+    email: Annotated[str, Meta(pattern=r"^[^@]+@[^@]+\.[^@]+$")]
+    bio: str = ""
+
+
+class BenchAuthorWithValidators(Serializer):
+    """Django-Bolt Serializer with custom field validators."""
+    id: int
+    name: Annotated[str, Meta(min_length=2)]
+    email: Annotated[str, Meta(pattern=r"^[^@]+@[^@]+\.[^@]+$")]
+    bio: str = ""
+
+    @field_validator("name")
+    def strip_name(cls, value: str) -> str:
+        """Strip whitespace from name."""
+        return value.strip()
+
+    @field_validator("email")
+    def lowercase_email(cls, value: str) -> str:
+        """Lowercase email for consistency."""
+        return value.lower()
+
+    # @field_validator("password")
+    # def validate_password(cls, value: str) -> str:
+    #     """Validate password strength."""
+    #     if value == "4321":
+    #         raise ValidationError("Incorrect password")
+    #     # MUST return the value (or transformed value)
+    #     return value
+
+@api.post("/bench/serializer-raw")
+async def bench_serializer_raw(author: BenchAuthorRaw) -> BenchAuthorRaw:
+    """
+    Benchmark endpoint using raw msgspec (no validators).
+    Tests pure msgspec deserialization and serialization.
+    """
+    return author
+
+
+@api.post("/bench/serializer-validated")
+async def bench_serializer_validated(author: BenchAuthorWithValidators) -> BenchAuthorWithValidators:
+    """
+    Benchmark endpoint using Django-Bolt Serializer with custom validators.
+    Tests deserialization with field validators (strip, lowercase).
+
+    Validates that:
+    - name is stripped of whitespace
+    - email is lowercased
+    """
+    # Ensure validations worked
+    assert author.name == author.name.strip(), "Name should be stripped"
+    assert author.email == author.email.lower(), "Email should be lowercase"
+
+    return author
+
+
 # ============================================================================
 # Class-Based Views (APIView) - Using Decorator Syntax
 # ============================================================================
@@ -695,9 +848,6 @@ class ItemAPIView(APIView):
 # Class-Based Views (ViewSet) - Using Unified ViewSet Pattern with @action
 # ============================================================================
 
-from django_bolt import action
-
-
 
 # ============================================================================
 # Benchmark ViewSets - Using Decorator Syntax
@@ -778,7 +928,7 @@ class StreamViewSet(ViewSet):
         def gen():
             for i in range(100):
                 yield "x"
-        return StreamingResponse(gen, media_type="text/plain")
+        return StreamingResponse(gen(), media_type="text/plain")
 
 
 @api.view("/cbv-sse")
@@ -791,7 +941,7 @@ class SSEViewSet(ViewSet):
         def gen():
             for i in range(3):
                 yield f"data: {i}\n\n"
-        return StreamingResponse(gen, media_type="text/event-stream")
+        return StreamingResponse(gen(), media_type="text/event-stream")
 
 
 @api.view("/cbv-chat-completions")
