@@ -17,14 +17,18 @@ class-based views (APIView, ViewSet, ModelViewSet):
 import pytest
 import msgspec
 from typing import Annotated
-from django_bolt import BoltAPI
+from django.contrib.auth.models import User, Permission
+from django.contrib.contenttypes.models import ContentType
+from django_bolt import BoltAPI, action
 from django_bolt.testing import TestClient
 from django_bolt.views import APIView, ViewSet, ModelViewSet
 from django_bolt.params import Query, Path, Body, Header, Cookie, Form, File, Depends
 from django_bolt.exceptions import HTTPException
 from django_bolt.responses import StreamingResponse, FileResponse
-from django_bolt.auth import JWTAuthentication, APIKeyAuthentication
-from django_bolt.auth.guards import IsAuthenticated, IsAdminUser, IsStaff, HasPermission
+from django_bolt.auth import JWTAuthentication, APIKeyAuthentication, IsAuthenticated
+from django_bolt.auth.guards import IsAdminUser, IsStaff, HasPermission
+from django_bolt.auth.jwt_utils import create_jwt_for_user, get_current_user
+from django_bolt.middleware import cors, rate_limit, skip_middleware
 
 
 # --- Fixtures ---
@@ -163,7 +167,7 @@ def test_header_parameter_extraction(api):
 
         # Missing required header
         response = client.get("/protected")
-        assert response.status_code == 500  # ValueError raises 500
+        assert response.status_code == 400  # Missing required header returns 400
 
 
 def test_cookie_parameter_extraction(api):
@@ -189,7 +193,7 @@ def test_cookie_parameter_extraction(api):
 
         # Missing required cookie
         response = client.get("/session")
-        assert response.status_code == 500  # ValueError raises 500
+        assert response.status_code == 400  # Missing required cookie returns 400
 
 
 def test_mixed_parameter_sources(api):
