@@ -8,11 +8,14 @@ Tests cover:
 - Request body validation errors
 - Type coercion and conversion
 """
+import json
 import pytest
 import msgspec
+
 from django_bolt import BoltAPI
-from django_bolt.exceptions import RequestValidationError
-from django_bolt.error_handlers import handle_exception
+from django_bolt.binding import create_body_extractor, convert_primitive, get_msgspec_decoder, _DECODER_CACHE
+from django_bolt.error_handlers import handle_exception, msgspec_validation_error_to_dict
+from django_bolt.exceptions import RequestValidationError, HTTPException
 
 
 class UserCreate(msgspec.Struct):
@@ -47,8 +50,6 @@ class TestInvalidJSONParsing:
 
     def test_invalid_json_syntax_returns_422(self):
         """Test that malformed JSON returns 422 with proper error."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         # Invalid JSON syntax
@@ -67,8 +68,6 @@ class TestInvalidJSONParsing:
 
     def test_empty_json_body_returns_422(self):
         """Test that empty JSON body returns 422."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         # Should convert DecodeError to RequestValidationError (422)
@@ -84,8 +83,6 @@ class TestInvalidJSONParsing:
 
     def test_non_json_content_returns_422(self):
         """Test that non-JSON content returns 422."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         # Plain text instead of JSON
@@ -102,8 +99,6 @@ class TestInvalidJSONParsing:
 
     def test_invalid_json_object_type(self):
         """Test that JSON with wrong root type fails validation."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         # Array instead of object
@@ -124,8 +119,6 @@ class TestMsgspecStructValidation:
 
     def test_missing_required_field(self):
         """Test that missing required field raises ValidationError."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         # Missing 'age' field
@@ -137,8 +130,6 @@ class TestMsgspecStructValidation:
 
     def test_wrong_field_type(self):
         """Test that wrong field type raises ValidationError."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         # age should be int, not string
@@ -151,8 +142,6 @@ class TestMsgspecStructValidation:
 
     def test_null_for_required_field(self):
         """Test that null for required field raises ValidationError."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         with pytest.raises(msgspec.ValidationError):
@@ -160,8 +149,6 @@ class TestMsgspecStructValidation:
 
     def test_extra_fields_allowed_by_default(self):
         """Test that extra fields are allowed by default in msgspec."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserCreate)
 
         # Should succeed even with extra field
@@ -172,8 +159,6 @@ class TestMsgspecStructValidation:
 
     def test_nested_struct_validation(self):
         """Test validation of nested structs."""
-        from django_bolt.binding import create_body_extractor
-
         extractor = create_body_extractor("user", UserWithNested)
 
         # Valid nested structure
@@ -206,7 +191,6 @@ class TestMsgspecStructValidation:
             name: str
             tags: list[str]
 
-        from django_bolt.binding import create_body_extractor
         extractor = create_body_extractor("user", UserWithTags)
 
         # Valid array
@@ -236,7 +220,6 @@ class TestProductionVsDebugMode:
         assert status == 422, "Validation error must return 422"
 
         # Parse response
-        import json
         data = json.loads(body)
 
         # Should have validation errors
@@ -269,7 +252,6 @@ class TestProductionVsDebugMode:
         prod_status, prod_headers, prod_body = handle_exception(exc, debug=False)
         assert prod_status == 500
 
-        import json
         prod_data = json.loads(prod_body)
         assert prod_data["detail"] == "Internal Server Error", \
             "Production should hide error details"
@@ -316,7 +298,6 @@ class TestRequestValidationErrorHandling:
 
         assert status == 422
 
-        import json
         data = json.loads(body)
 
         # Should return errors in detail field
@@ -342,8 +323,6 @@ class TestRequestValidationErrorHandling:
 
     def test_msgspec_error_to_request_validation_error(self):
         """Test that msgspec.ValidationError is converted properly."""
-        from django_bolt.error_handlers import msgspec_validation_error_to_dict
-
         # Create a validation error
         class TestStruct(msgspec.Struct):
             name: str
@@ -369,8 +348,6 @@ class TestTypeCoercionEdgeCases:
 
     def test_boolean_coercion_from_string(self):
         """Test boolean coercion from string query params."""
-        from django_bolt.binding import convert_primitive
-
         # True values
         assert convert_primitive("true", bool) is True
         assert convert_primitive("True", bool) is True
@@ -387,9 +364,6 @@ class TestTypeCoercionEdgeCases:
 
     def test_number_coercion_errors(self):
         """Test that invalid number strings raise errors."""
-        from django_bolt.binding import convert_primitive
-        from django_bolt.exceptions import HTTPException
-
         # Invalid int - now raises HTTPException(422) instead of ValueError
         with pytest.raises(HTTPException) as exc_info:
             convert_primitive("not_a_number", int)
@@ -402,9 +376,6 @@ class TestTypeCoercionEdgeCases:
 
     def test_empty_string_coercion(self):
         """Test coercion of empty strings."""
-        from django_bolt.binding import convert_primitive
-        from django_bolt.exceptions import HTTPException
-
         # Empty string for string type should be empty string
         assert convert_primitive("", str) == ""
 
@@ -419,7 +390,6 @@ class TestTypeCoercionEdgeCases:
             name: str
             email: str | None = None
 
-        from django_bolt.binding import create_body_extractor
         extractor = create_body_extractor("user", UserOptional)
 
         # Explicit null
@@ -438,8 +408,6 @@ class TestJSONParsingPerformance:
 
     def test_decoder_caching(self):
         """Test that msgspec decoders are cached for performance."""
-        from django_bolt.binding import get_msgspec_decoder, _DECODER_CACHE
-
         # Clear cache
         _DECODER_CACHE.clear()
 
@@ -453,8 +421,6 @@ class TestJSONParsingPerformance:
 
     def test_large_json_parsing(self):
         """Test parsing of large JSON payloads."""
-        from django_bolt.binding import create_body_extractor
-
         class LargeStruct(msgspec.Struct):
             items: list[dict]
 
@@ -462,7 +428,6 @@ class TestJSONParsingPerformance:
 
         # Create large JSON with 1000 items
         items = [{"id": i, "name": f"item_{i}"} for i in range(1000)]
-        import json
         large_json = json.dumps({"items": items}).encode()
 
         # Should parse successfully
@@ -480,7 +445,6 @@ class TestJSONParsingPerformance:
         class Level1(msgspec.Struct):
             level2: Level2
 
-        from django_bolt.binding import create_body_extractor
         extractor = create_body_extractor("data", Level1)
 
         nested_json = b'''{
@@ -526,7 +490,6 @@ class TestIntegrationWithBoltAPI:
         status, headers, body = handle_exception(exc, debug=False)
         assert status == 422
 
-        import json
         data = json.loads(body)
         assert len(data["detail"]) == 1
         assert data["detail"][0]["loc"] == ["body", "age"]
