@@ -39,9 +39,12 @@ ab -k -c $C -n $N http://$HOST:$PORT/ 2>/dev/null | grep -E "(Requests per secon
 echo ""
 echo "## 10kb JSON Response Performance"
 
-printf "### 10kb JSON  (/10k-json)\n"
+printf "### 10kb JSON (Async) (/10k-json)\n"
 ab -k -c $C -n $N http://$HOST:$PORT/10k-json 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
 
+printf "### 10kb JSON (Sync) (/sync-10k-json)\n"
+ab -k -c $C -n $N http://$HOST:$PORT/sync-10k-json 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+
 echo ""
 echo "## Response Type Endpoints"
 
@@ -63,42 +66,107 @@ ab -k -c $C -n $N -r -H 'Accept: */*' http://$HOST:$PORT/redirect 2>/dev/null |
 printf "### File Static via FileResponse (/file-static)\n"
 ab -k -c $C -n $N http://$HOST:$PORT/file-static 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
 
+echo ""
+echo "## Authentication & Authorization Performance"
+
+# Create a Django user and JWT token for testing
+TOKEN=$(uv run python << 'PYTHON_TOKEN_SCRIPT'
+import os
+import sys
+import jwt
+import time
+
+os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'testproject.settings')
+
+try:
+    import django
+    django.setup()
+
+    from django.conf import settings
+    from django.contrib.auth.models import User
+
+    # Get or create a test user
+    user, created = User.objects.get_or_create(
+        username='benchuser',
+        defaults={'email': 'bench@example.com'}
+    )
+
+    # Create JWT token with correct user ID
+    payload = {
+        'sub': str(user.id),
+        'exp': int(time.time()) + 3600,
+        'iat': int(time.time()),
+        'is_staff': user.is_staff,
+        'is_superuser': user.is_superuser,
+        'username': user.username,
+        'email': user.email
+    }
+
+    token = jwt.encode(payload, settings.SECRET_KEY, algorithm='HS256')
+    print(token)
+except Exception as e:
+    print(f"Error: {e}", file=sys.stderr)
+    sys.exit(1)
+PYTHON_TOKEN_SCRIPT
+2>/dev/null)
+
+# Only run auth tests if we have a valid token
+if [ -n "$TOKEN" ] && [ ${#TOKEN} -gt 50 ]; then
+    printf "### Get Authenticated User (/auth/me)\n"
+    AUTH_HEADER="Authorization: Bearer $TOKEN"
+    ab -k -c $C -n $N -H "$AUTH_HEADER" http://$HOST:$PORT/auth/me 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+
+    printf "### Get User via Dependency (/auth/me-dependency)\n"
+    ab -k -c $C -n $N -H "$AUTH_HEADER" http://$HOST:$PORT/auth/me-dependency 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+
+    printf "### Get Auth Context (/auth/context) validated jwt no db\n"
+    ab -k -c $C -n $N -H "$AUTH_HEADER" http://$HOST:$PORT/auth/context 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+else
+    echo "Skipped auth benchmarks: Could not generate JWT token"
+fi
+
 # Streaming and SSE tests using hey (better than ab for streaming)
 echo ""
 echo "## Streaming and SSE Performance"
-
+echo "SEE STREAMING_BENCHMARK_DEV.md"
 # Check if hey is available
-HEY_BIN=""
-if command -v hey &> /dev/null; then
-    HEY_BIN="hey"
-elif [ -f "$HOME/go/bin/hey" ]; then
-    HEY_BIN="$HOME/go/bin/hey"
-elif [ -f "$HOME/.local/bin/hey" ]; then
-    HEY_BIN="$HOME/.local/bin/hey"
-fi
+# HEY_BIN=""
+# if command -v hey &> /dev/null; then
+#     HEY_BIN="hey"
+# elif [ -f "$HOME/go/bin/hey" ]; then
+#     HEY_BIN="$HOME/go/bin/hey"
+# elif [ -f "$HOME/.local/bin/hey" ]; then
+#     HEY_BIN="$HOME/.local/bin/hey"
+# fi
 
-if [ -n "$HEY_BIN" ]; then
-    printf "### Streaming Plain Text (/stream)\n"
-    timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C http://$HOST:$PORT/stream 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(stream timed out after ${HEY_TIMEOUT}s)"
-    
-    printf "### Server-Sent Events (/sse)\n"
-    timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Accept: text/event-stream" http://$HOST:$PORT/sse 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(sse timed out after ${HEY_TIMEOUT}s)"
+# if [ -n "$HEY_BIN" ]; then
+#     printf "### Streaming Plain Text (Async) (/stream)\n"
+#     timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C http://$HOST:$PORT/stream 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(stream timed out after ${HEY_TIMEOUT}s)"
 
-    printf "### Server-Sent Events (async) (/sse-async)\n"
-    timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Accept: text/event-stream" http://$HOST:$PORT/sse-async 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(sse-async timed out after ${HEY_TIMEOUT}s)"
+#     printf "### Streaming Plain Text (Sync) (/sync-stream)\n"
+#     timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C http://$HOST:$PORT/sync-stream 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(sync-stream timed out after ${HEY_TIMEOUT}s)"
 
-    printf "### OpenAI Chat Completions (stream) (/v1/chat/completions)\n"
-    BODY_STREAM='{"model":"gpt-4o-mini","messages":[{"role":"user","content":"Say hi"}],"stream":true,"n_chunks":50,"token":" hi","delay_ms":0}'
-    echo "$BODY_STREAM" > /tmp/bolt_chat_stream.json
-    timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Content-Type: application/json" -m POST -D /tmp/bolt_chat_stream.json http://$HOST:$PORT/v1/chat/completions 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Bytes In|Bytes Out|Status code distribution:)" | head -15 || echo "(chat stream timed out after ${HEY_TIMEOUT}s)"
+#     printf "### Server-Sent Events (Async) (/sse)\n"
+#     timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Accept: text/event-stream" http://$HOST:$PORT/sse 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(sse timed out after ${HEY_TIMEOUT}s)"
 
-    printf "### OpenAI Chat Completions (async stream) (/v1/chat/completions-async)\n"
-    BODY_STREAM_ASYNC='{"model":"gpt-4o-mini","messages":[{"role":"user","content":"Say hi"}],"stream":true,"n_chunks":50,"token":" hi","delay_ms":0}'
-    echo "$BODY_STREAM_ASYNC" > /tmp/bolt_chat_stream_async.json
-    timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Content-Type: application/json" -m POST -D /tmp/bolt_chat_stream_async.json http://$HOST:$PORT/v1/chat/completions-async 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Bytes In|Bytes Out|Status code distribution:)" | head -15 || echo "(chat async stream timed out after ${HEY_TIMEOUT}s)"
-else
-    echo "hey not installed. Run: ./scripts/install_hey.sh"
-fi
+#     printf "### Server-Sent Events (Sync) (/sync-sse)\n"
+#     timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Accept: text/event-stream" http://$HOST:$PORT/sync-sse 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(sync-sse timed out after ${HEY_TIMEOUT}s)"
+
+#     printf "### Server-Sent Events (Async Generator) (/sse-async)\n"
+#     timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Accept: text/event-stream" http://$HOST:$PORT/sse-async 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Status code distribution:)" | head -10 || echo "(sse-async timed out after ${HEY_TIMEOUT}s)"
+
+#     printf "### OpenAI Chat Completions (stream) (/v1/chat/completions)\n"
+#     BODY_STREAM='{"model":"gpt-4o-mini","messages":[{"role":"user","content":"Say hi"}],"stream":true,"n_chunks":50,"token":" hi","delay_ms":0}'
+#     echo "$BODY_STREAM" > /tmp/bolt_chat_stream.json
+#     timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Content-Type: application/json" -m POST -D /tmp/bolt_chat_stream.json http://$HOST:$PORT/v1/chat/completions 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Bytes In|Bytes Out|Status code distribution:)" | head -15 || echo "(chat stream timed out after ${HEY_TIMEOUT}s)"
+
+#     printf "### OpenAI Chat Completions (async stream) (/v1/chat/completions-async)\n"
+#     BODY_STREAM_ASYNC='{"model":"gpt-4o-mini","messages":[{"role":"user","content":"Say hi"}],"stream":true,"n_chunks":50,"token":" hi","delay_ms":0}'
+#     echo "$BODY_STREAM_ASYNC" > /tmp/bolt_chat_stream_async.json
+#     timeout "$HEY_TIMEOUT" $HEY_BIN -n $N -c $C -H "Content-Type: application/json" -m POST -D /tmp/bolt_chat_stream_async.json http://$HOST:$PORT/v1/chat/completions-async 2>&1 | grep -E "(Requests/sec:|Total:|Fastest:|Slowest:|Average:|Bytes In|Bytes Out|Status code distribution:)" | head -15 || echo "(chat async stream timed out after ${HEY_TIMEOUT}s)"
+# else
+#     echo "hey not installed. Run: ./scripts/install_hey.sh"
+# fi
 
 # Additional endpoint: GET /items/{item_id}
 echo ""
@@ -142,12 +210,40 @@ if [ "$UCODE" != "200" ]; then
   exit 1
 fi
 
-echo "### Users Full10 (/users/full10)"
+# Seed users for benchmarking (create 1000 test users)
+echo "Seeding 1000 users for benchmark..."
+SEED_CODE=$(curl -s -o /dev/null -w '%{http_code}' -X GET http://$HOST:$PORT/users/seed?count=1000)
+if [ "$SEED_CODE" != "200" ]; then
+  echo "Warning: Failed to seed users (got $SEED_CODE), benchmarking with empty database" >&2
+else
+  echo "Successfully seeded users"
+
+  # Validate users exist by checking /users/full10
+  USERS_RESPONSE=$(curl -s http://$HOST:$PORT/users/full10)
+  USER_COUNT=$(echo "$USERS_RESPONSE" | grep -o '"id"' | wc -l)
+  if [ "$USER_COUNT" -eq 0 ]; then
+    echo "Warning: No users found after seeding, benchmarking with empty database" >&2
+  else
+    echo "Validated: $USER_COUNT users exist in database"
+  fi
+fi
+
+echo "### Users Full10 (Async) (/users/full10)"
 ab -k -c $C -n $N http://$HOST:$PORT/users/full10 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
 
-echo "### Users Mini10 (/users/mini10)"
+echo "### Users Full10 (Sync) (/users/sync-full10)"
+ab -k -c $C -n $N http://$HOST:$PORT/users/sync-full10 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+
+echo "### Users Mini10 (Async) (/users/mini10)"
 ab -k -c $C -n $N http://$HOST:$PORT/users/mini10 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
 
+echo "### Users Mini10 (Sync) (/users/sync-mini10)"
+ab -k -c $C -n $N http://$HOST:$PORT/users/sync-mini10 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+
+# Clean up: delete all users
+echo "Cleaning up test users..."
+curl -s -X POST http://$HOST:$PORT/users/delete >/dev/null 2>&1
+
 kill -TERM -$SERVER_PID 2>/dev/null || true
 pkill -TERM -f "manage.py runbolt --host $HOST --port $PORT" 2>/dev/null || true
 
@@ -231,6 +327,24 @@ fi
 echo ""
 echo "## ORM Performance with CBV"
 
+# Seed users for CBV benchmarking
+echo "Seeding 1000 users for CBV benchmark..."
+SEED_CODE=$(curl -s -o /dev/null -w '%{http_code}' -X POST http://$HOST:$PORT/users/seed?count=1000)
+if [ "$SEED_CODE" != "200" ]; then
+  echo "Warning: Failed to seed users (got $SEED_CODE), benchmarking with empty database" >&2
+else
+  echo "Successfully seeded users"
+
+  # Validate users exist by checking /users/cbv-mini10
+  USERS_RESPONSE=$(curl -s http://$HOST:$PORT/users/cbv-mini10)
+  USER_COUNT=$(echo "$USERS_RESPONSE" | grep -o '"id"' | wc -l)
+  if [ "$USER_COUNT" -eq 0 ]; then
+    echo "Warning: No users found after seeding, benchmarking with empty database" >&2
+  else
+    echo "Validated: $USER_COUNT users exist in database"
+  fi
+fi
+
 # Sanity check
 UCODE=$(curl -s -o /dev/null -w '%{http_code}' http://$HOST:$PORT/users/cbv-mini10)
 if [ "$UCODE" != "200" ]; then
@@ -240,6 +354,9 @@ else
   ab -k -c $C -n $N http://$HOST:$PORT/users/cbv-mini10 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
 fi
 
+# Clean up: delete all users
+echo "Cleaning up test users..."
+curl -s -X POST http://$HOST:$PORT/users/delete >/dev/null 2>&1
 
 echo ""
 
@@ -334,8 +451,62 @@ if [ "$PCODE" != "200" ]; then
 else
   ab -k -c $C -n $N -p "$BODY_FILE" -T 'application/json' http://$HOST:$PORT/bench/parse 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
 fi
-kill -TERM -$SERVER_PID 2>/dev/null || true
-pkill -TERM -f "manage.py runbolt --host $HOST --port $PORT" 2>/dev/null || true
 rm -f "$BODY_FILE"
 
+echo ""
+echo "## Serializer Performance Benchmarks"
+
+# Test raw msgspec (baseline)
+SERIALIZER_RAW=$(mktemp)
+cat > "$SERIALIZER_RAW" << 'JSON'
+{
+  "id": 1,
+  "name": "John Doe",
+  "email": "john@example.com",
+  "bio": "Software developer"
+}
+JSON
+
+echo "### Raw msgspec Serializer (POST /bench/serializer-raw)"
+ab -k -c $C -n $N -p "$SERIALIZER_RAW" -T 'application/json' http://$HOST:$PORT/bench/serializer-raw 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+rm -f "$SERIALIZER_RAW"
+
+# Test with custom validators
+SERIALIZER_VALIDATED=$(mktemp)
+cat > "$SERIALIZER_VALIDATED" << 'JSON'
+{
+  "id": 1,
+  "name": "  John Doe  ",
+  "email": "JOHN@EXAMPLE.COM",
+  "bio": "Software developer"
+}
+JSON
+
+echo "### Django-Bolt Serializer with Validators (POST /bench/serializer-validated)"
+ab -k -c $C -n $N -p "$SERIALIZER_VALIDATED" -T 'application/json' http://$HOST:$PORT/bench/serializer-validated 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+rm -f "$SERIALIZER_VALIDATED"
+
+# Test users endpoint with raw msgspec
+USER_BENCH=$(mktemp)
+cat > "$USER_BENCH" << 'JSON'
+{
+  "id": 1,
+  "username": "testuser",
+  "email": "test@example.com",
+  "bio": "Test bio"
+}
+JSON
+
+echo "### Users msgspec Serializer (POST /users/bench/msgspec)"
+USCODE=$(curl -s -o /dev/null -w '%{http_code}' -X POST -H 'Content-Type: application/json' --data-binary @"$USER_BENCH" http://$HOST:$PORT/users/bench/msgspec)
+if [ "$USCODE" = "200" ]; then
+  ab -k -c $C -n $N -p "$USER_BENCH" -T 'application/json' http://$HOST:$PORT/users/bench/msgspec 2>/dev/null | grep -E "(Requests per second|Time per request|Failed requests)"
+else
+  echo "Skipped: Users msgspec endpoint returned $USCODE" >&2
+fi
+rm -f "$USER_BENCH"
+
+kill -TERM -$SERVER_PID 2>/dev/null || true
+pkill -TERM -f "manage.py runbolt --host $HOST --port $PORT" 2>/dev/null || true
+
 
