@@ -5,11 +5,11 @@ use bytes::Bytes;
 use futures_util::stream;
 use pyo3::prelude::*;
 use pyo3::types::{PyBytes, PyDict, PyTuple};
+use std::io::ErrorKind;
 use std::sync::Arc;
 use tokio::fs::File;
 use tokio::io::AsyncReadExt;
 
-use crate::direct_stream;
 use crate::error;
 use crate::metadata::CorsConfig;
 use crate::middleware;
@@ -17,7 +17,7 @@ use crate::middleware::auth::populate_auth_context;
 use crate::request::PyRequest;
 use crate::router::parse_query_string;
 use crate::state::{AppState, GLOBAL_ROUTER, ROUTE_METADATA, TASK_LOCALS};
-use crate::streaming::create_python_stream;
+use crate::streaming::{create_python_stream, create_sse_stream};
 use crate::validation::{parse_cookies_inline, validate_auth_and_guards, AuthGuardResult};
 
 // Reuse the global Python asyncio event loop created at server startup (TASK_LOCALS)
@@ -347,9 +347,9 @@ pub async fn handle_request(
     // Check if this is a HEAD request (needed for body stripping after Python handler)
     let is_head_request = method == "HEAD";
 
-    // Single GIL acquisition for all Python operations
+    // All handlers (sync and async) go through async dispatch path
+    // Sync handlers are executed in thread pool via sync_to_thread() in Python layer
     let fut = match Python::attach(|py| -> PyResult<_> {
-        // Clone Python objects
         let dispatch = state.dispatch.clone_ref(py);
         let handler = route_handler.clone_ref(py);
 
@@ -372,6 +372,7 @@ pub async fn handle_request(
             headers,
             cookies,
             context,
+            user: None,
         };
         let request_obj = Py::new(py, request)?;
 
@@ -380,15 +381,13 @@ pub async fn handle_request(
             pyo3::exceptions::PyRuntimeError::new_err("Asyncio loop not initialized")
         })?;
 
-        // Pass handler_id to dispatch so it can lookup the original API instance
+        // Call dispatch (always returns a coroutine since _dispatch is async)
         let coroutine = dispatch.call1(py, (handler, request_obj, handler_id))?;
         pyo3_async_runtimes::into_future_with_locals(locals, coroutine.into_bound(py))
     }) {
         Ok(f) => f,
         Err(e) => {
-            // Use new error handler
             return Python::attach(|py| {
-                // Convert PyErr to exception instance
                 e.restore(py);
                 if let Some(exc) = PyErr::take(py) {
                     let exc_value = exc.value(py);
@@ -553,7 +552,6 @@ pub async fn handle_request(
                         }
                         Err(e) => {
                             // Return appropriate HTTP status based on error kind
-                            use std::io::ErrorKind;
                             match e.kind() {
                                 ErrorKind::NotFound => HttpResponse::NotFound()
                                     .content_type("text/plain; charset=utf-8")
@@ -691,7 +689,6 @@ pub async fn handle_request(
                                 builder.body(response_body)
                             }
                             Err(e) => {
-                                use std::io::ErrorKind;
                                 match e.kind() {
                                     ErrorKind::NotFound => HttpResponse::NotFound()
                                         .content_type("text/plain; charset=utf-8")
@@ -771,10 +768,15 @@ pub async fn handle_request(
                         Ok(c) => c.unbind(),
                         Err(_) => return None,
                     };
-                    Some((status_code, headers, media_type, content_obj))
+                    // Extract pre-computed is_async_generator metadata (detected at StreamingResponse instantiation)
+                    let is_async_generator: bool = obj
+                        .getattr("is_async_generator")
+                        .and_then(|v| v.extract())
+                        .unwrap_or(false);
+                    Some((status_code, headers, media_type, content_obj, is_async_generator))
                 });
 
-                if let Some((status_code, headers, media_type, content_obj)) = streaming {
+                if let Some((status_code, headers, media_type, content_obj, is_async_generator)) = streaming {
                     let status = StatusCode::from_u16(status_code).unwrap_or(StatusCode::OK);
                     let mut builder = HttpResponse::build(status);
                     for (k, v) in headers {
@@ -797,72 +799,21 @@ pub async fn handle_request(
                             return builder.body(Vec::<u8>::new());
                         }
 
-                        let mut final_content_obj = content_obj;
-                        // Combine async check and wrapping into single GIL acquisition
-                        let (is_async_sse, wrapped_content) = Python::attach(|py| {
-                            let obj = final_content_obj.bind(py);
-                            let has_async = obj.hasattr("__aiter__").unwrap_or(false)
-                                || obj.hasattr("__anext__").unwrap_or(false);
-                            if !has_async {
-                                return (false, None);
-                            }
-                            // Try to wrap async iterator
-                            let wrapped = (|| -> Option<Py<PyAny>> {
-                                let collector_module =
-                                    py.import("django_bolt.async_collector").ok()?;
-                                let collector_class =
-                                    collector_module.getattr("AsyncToSyncCollector").ok()?;
-                                collector_class
-                                    .call1((obj.clone(), 5, 1))
-                                    .ok()
-                                    .map(|w| w.unbind())
-                            })();
-                            (wrapped.is_none(), wrapped)
-                        });
-                        if let Some(w) = wrapped_content {
-                            final_content_obj = w;
-                        }
-                        if is_async_sse {
-                            builder.append_header(("X-Accel-Buffering", "no"));
-                            builder.append_header((
-                                "Cache-Control",
-                                "no-cache, no-store, must-revalidate",
-                            ));
-                            builder.append_header(("Pragma", "no-cache"));
-                            builder.append_header(("Expires", "0"));
-                            if skip_compression {
-                                builder.append_header(("Content-Encoding", "identity"));
-                            }
-                            builder.content_type("text/event-stream");
-                            return builder.streaming(create_python_stream(final_content_obj));
-                        } else {
-                            match direct_stream::create_sse_response(final_content_obj) {
-                                Ok(mut resp) => {
-                                    if skip_compression {
-                                        if let Ok(val) = HeaderValue::try_from("identity") {
-                                            resp.headers_mut().insert(
-                                                actix_web::http::header::CONTENT_ENCODING,
-                                                val,
-                                            );
-                                        }
-                                    }
-                                    return resp;
-                                }
-                                Err(_) => {
-                                    builder.append_header(("X-Accel-Buffering", "no"));
-                                    builder.append_header((
-                                        "Cache-Control",
-                                        "no-cache, no-store, must-revalidate",
-                                    ));
-                                    builder.append_header(("Pragma", "no-cache"));
-                                    builder.append_header(("Expires", "0"));
-                                    if skip_compression {
-                                        builder.append_header(("Content-Encoding", "identity"));
-                                    }
-                                    return builder.content_type("text/event-stream").body("");
-                                }
-                            }
+                        let final_content_obj = content_obj;
+                        builder.append_header(("X-Accel-Buffering", "no"));
+                        builder.append_header((
+                            "Cache-Control",
+                            "no-cache, no-store, must-revalidate",
+                        ));
+                        builder.append_header(("Pragma", "no-cache"));
+                        builder.append_header(("Expires", "0"));
+                        if skip_compression {
+                            builder.append_header(("Content-Encoding", "identity"));
                         }
+                        builder.content_type("text/event-stream");
+
+                        let stream = create_sse_stream(final_content_obj, is_async_generator);
+                        return builder.streaming(stream);
                     } else {
                         // HEAD requests must have empty body per RFC 7231
                         if is_head_request {
@@ -872,64 +823,24 @@ pub async fn handle_request(
                             return builder.body(Vec::<u8>::new());
                         }
 
-                        let mut final_content = content_obj;
-                        // Combine async check and wrapping into single GIL acquisition
-                        let (needs_async_stream, wrapped_content) = Python::attach(|py| {
-                            let obj = final_content.bind(py);
-                            let has_async = obj.hasattr("__aiter__").unwrap_or(false)
-                                || obj.hasattr("__anext__").unwrap_or(false);
-                            if !has_async {
-                                return (false, None);
-                            }
-                            // Try to wrap async iterator
-                            let wrapped = (|| -> Option<Py<PyAny>> {
-                                let collector_module =
-                                    py.import("django_bolt.async_collector").ok()?;
-                                let collector_class =
-                                    collector_module.getattr("AsyncToSyncCollector").ok()?;
-                                collector_class
-                                    .call1((obj.clone(), 20, 2))
-                                    .ok()
-                                    .map(|w| w.unbind())
-                            })();
-                            (wrapped.is_none(), wrapped)
-                        });
-
-                        if needs_async_stream {
-                            if skip_compression {
-                                builder.append_header(("Content-Encoding", "identity"));
-                            }
-                            let stream = create_python_stream(final_content);
-                            return builder.streaming(stream);
-                        }
-
-                        if let Some(w) = wrapped_content {
-                            final_content = w;
-                        }
-                        {
-                            let mut direct = direct_stream::PythonDirectStream::new(final_content);
-                            if let Some(body) = direct.try_collect_small() {
-                                if skip_compression {
-                                    builder.append_header(("Content-Encoding", "identity"));
-                                }
-                                return builder.body(body);
-                            }
-                            if skip_compression {
-                                builder.append_header(("Content-Encoding", "identity"));
-                            }
-                            return builder.streaming(Box::pin(direct));
+                        let final_content = content_obj;
+                        // Use unified streaming for all streaming responses (sync and async)
+                        if skip_compression {
+                            builder.append_header(("Content-Encoding", "identity"));
                         }
+                        let stream = create_python_stream(final_content, is_async_generator);
+                        return builder.streaming(stream);
                     }
                 } else {
                     return Python::attach(|py| {
                         error::build_error_response(
-                            py,
-                            500,
-                            "Handler returned unsupported response type (expected tuple or StreamingResponse)".to_string(),
-                            vec![],
-                            None,
-                            state.debug,
-                        )
+                        py,
+                        500,
+                        "Handler returned unsupported response type (expected tuple or StreamingResponse)".to_string(),
+                        vec![],
+                        None,
+                        state.debug,
+                    )
                     });
                 }
             }
