@@ -1,12 +1,13 @@
 use actix_web::web::Bytes;
+use futures_util::future::join_all;
 use futures_util::{stream, Stream};
 use pyo3::prelude::*;
 use pyo3::types::{PyByteArray, PyBytes, PyMemoryView, PyString};
 use std::pin::Pin;
-use std::time::Instant;
+use std::sync::atomic::Ordering;
 use tokio::sync::mpsc;
 
-use crate::state::TASK_LOCALS;
+use crate::state::{get_max_sync_streaming_threads, ACTIVE_SYNC_STREAMING_THREADS, TASK_LOCALS};
 // Streaming uses direct_stream only in higher-level handler; not directly here
 
 // Buffer pool imports removed (unused)
@@ -51,16 +52,11 @@ pub fn convert_python_chunk(value: &Bound<'_, PyAny>) -> Option<Bytes> {
     None
 }
 
+/// Create a stream with default batch sizes from environment
 pub fn create_python_stream(
     content: Py<PyAny>,
+    is_async_generator: bool,
 ) -> Pin<Box<dyn Stream<Item = Result<Bytes, std::io::Error>> + Send>> {
-    let debug_timing = std::env::var("DJANGO_BOLT_DEBUG_TIMING").is_ok();
-
-    let channel_capacity: usize = std::env::var("DJANGO_BOLT_STREAM_CHANNEL_CAPACITY")
-        .ok()
-        .and_then(|v| v.parse::<usize>().ok())
-        .filter(|&n| n > 0)
-        .unwrap_or(32);
     let batch_size: usize = std::env::var("DJANGO_BOLT_STREAM_BATCH_SIZE")
         .ok()
         .and_then(|v| v.parse::<usize>().ok())
@@ -71,66 +67,48 @@ pub fn create_python_stream(
         .and_then(|v| v.parse::<usize>().ok())
         .filter(|&n| n > 0)
         .unwrap_or(5);
+    create_python_stream_with_config(content, batch_size, sync_batch_size, is_async_generator)
+}
+
+/// Create a stream for SSE that sends items immediately (batch_size=1)
+pub fn create_sse_stream(
+    content: Py<PyAny>,
+    is_async_generator: bool,
+) -> Pin<Box<dyn Stream<Item = Result<Bytes, std::io::Error>> + Send>> {
+    create_python_stream_with_config(content, 1, 1, is_async_generator)
+}
+
+/// Internal function with configurable batch sizes
+fn create_python_stream_with_config(
+    content: Py<PyAny>,
+    async_batch_size: usize,
+    sync_batch_size: usize,
+    is_async_from_metadata: bool,
+) -> Pin<Box<dyn Stream<Item = Result<Bytes, std::io::Error>> + Send>> {
+    let channel_capacity: usize = std::env::var("DJANGO_BOLT_STREAM_CHANNEL_CAPACITY")
+        .ok()
+        .and_then(|v| v.parse::<usize>().ok())
+        .filter(|&n| n > 0)
+        .unwrap_or(32);
     let fast_path_threshold: usize = std::env::var("DJANGO_BOLT_STREAM_FAST_PATH_THRESHOLD")
         .ok()
         .and_then(|v| v.parse::<usize>().ok())
         .filter(|&n| n > 0)
         .unwrap_or(10);
+    // Note: content is guaranteed to be a generator instance (not a callable)
+    // because StreamingResponse validates this in Python at instantiation time.
+    // The is_async_generator flag was pre-computed from Python's inspect.
+    let resolved_target = Python::attach(|py| content.clone_ref(py));
+    let is_async_iter = is_async_from_metadata;
 
-    let resolve_start = if debug_timing {
-        Some(Instant::now())
-    } else {
-        None
-    };
-    let (resolved_target, is_async_iter) = Python::attach(|py| {
-        let mut target = content.clone_ref(py);
-        let obj = content.bind(py);
-        if obj.is_callable() {
-            if let Ok(new_obj) = obj.call0() {
-                target = new_obj.unbind();
-            }
-        }
-        let b = target.bind(py);
-        let has_async =
-            b.hasattr("__aiter__").unwrap_or(false) || b.hasattr("__anext__").unwrap_or(false);
-        (target, has_async)
-    });
-
-    if let Some(start) = resolve_start {
-        eprintln!(
-            "[TIMING] Iterator resolution: {:?}, is_async={}, batch_size={}, fast_path_threshold={}",
-            start.elapsed(),
-            is_async_iter,
-            if is_async_iter { batch_size } else { sync_batch_size },
-            fast_path_threshold
-        );
-    }
 
     let (tx, rx) = mpsc::channel::<Result<Bytes, std::io::Error>>(channel_capacity);
     let resolved_target_final = Python::attach(|py| resolved_target.clone_ref(py));
     let is_async_final = is_async_iter;
 
     if is_async_final {
-        let debug_async = debug_timing;
-        let async_batch_size = batch_size;
         let fast_path = fast_path_threshold;
         tokio::spawn(async move {
-            use futures_util::future::join_all;
-            let mut chunk_count = 0u32;
-            let mut total_gil_time = std::time::Duration::ZERO;
-            let mut total_await_time = std::time::Duration::ZERO;
-            let mut total_send_time = std::time::Duration::ZERO;
-            let task_start = if debug_async {
-                Some(Instant::now())
-            } else {
-                None
-            };
-            let init_start = if debug_async {
-                Some(Instant::now())
-            } else {
-                None
-            };
-
             let is_optimized_batcher = Python::attach(|py| {
                 if let Ok(name) = resolved_target_final.bind(py).get_type().name() {
                     name.to_string().contains("OptimizedStreamBatcher")
@@ -141,14 +119,6 @@ pub fn create_python_stream(
 
             let async_iter: Option<Py<PyAny>> = Python::attach(|py| {
                 let b = resolved_target_final.bind(py);
-                if debug_async {
-                    eprintln!(
-                        "[DEBUG] Checking async iterator: has __aiter__={}, has __anext__={}, is_optimized={}",
-                        b.hasattr("__aiter__").unwrap_or(false),
-                        b.hasattr("__anext__").unwrap_or(false),
-                        is_optimized_batcher
-                    );
-                }
                 if b.hasattr("__aiter__").unwrap_or(false) {
                     match b.call_method0("__aiter__") {
                         Ok(it) => Some(it.unbind()),
@@ -161,12 +131,6 @@ pub fn create_python_stream(
                 }
             });
 
-            if let Some(start) = init_start {
-                eprintln!(
-                    "[TIMING] Async iterator initialization: {:?}",
-                    start.elapsed()
-                );
-            }
 
             if async_iter.is_none() {
                 let _ = tx
@@ -181,16 +145,10 @@ pub fn create_python_stream(
 
             let mut exhausted = false;
             let mut batch_futures = Vec::with_capacity(async_batch_size);
-            let mut batch_count = 0usize;
             let mut consecutive_small_batches = 0u8;
             let mut current_batch_size = std::cmp::min(async_batch_size, fast_path);
 
             while !exhausted {
-                let batch_start = if debug_async {
-                    Some(Instant::now())
-                } else {
-                    None
-                };
                 batch_futures.clear();
                 Python::attach(|py| {
                     // Reuse the global event loop locals initialized at server startup
@@ -230,15 +188,6 @@ pub fn create_python_stream(
                     }
                 });
 
-                if let Some(start) = batch_start {
-                    eprintln!(
-                        "[TIMING] Batch {} future collection ({} futures, target={}): {:?}",
-                        batch_count,
-                        batch_futures.len(),
-                        current_batch_size,
-                        start.elapsed()
-                    );
-                }
 
                 if batch_futures.len() < current_batch_size / 2 {
                     consecutive_small_batches += 1;
@@ -257,28 +206,8 @@ pub fn create_python_stream(
                     break;
                 }
 
-                let await_start = if debug_async {
-                    Some(Instant::now())
-                } else {
-                    None
-                };
                 let results = join_all(batch_futures.drain(..)).await;
-                if let Some(start) = await_start {
-                    eprintln!(
-                        "[TIMING] Batch {} await ({} futures): {:?}",
-                        batch_count,
-                        results.len(),
-                        start.elapsed()
-                    );
-                    total_await_time += start.elapsed();
-                }
 
-                let convert_start = if debug_async {
-                    Some(Instant::now())
-                } else {
-                    None
-                };
-                let mut send_count = 0;
                 let mut got_stop_iteration = false;
                 for result in results {
                     match result {
@@ -297,11 +226,31 @@ pub fn create_python_stream(
                             });
                             if let Some(bytes) = bytes_opt {
                                 if tx.send(Ok(bytes)).await.is_err() {
+                                    // Client disconnected - close the async generator to run cleanup code
+                                    let close_result = Python::attach(|py| {
+                                        let iter_bound = async_iter.bind(py);
+                                        if iter_bound.hasattr("aclose").unwrap_or(false) {
+                                            if let Ok(awaitable) = iter_bound.call_method0("aclose") {
+                                                if let Some(locals) = TASK_LOCALS.get() {
+                                                    return pyo3_async_runtimes::into_future_with_locals(locals, awaitable).ok();
+                                                } else {
+                                                    eprintln!("[SSE WARNING] Unable to get task locals for async generator cleanup on disconnect");
+                                                }
+                                            } else {
+                                                eprintln!("[SSE WARNING] Failed to call aclose() on async generator during disconnect cleanup");
+                                            }
+                                        }
+                                        None
+                                    });
+                                    // Await the cleanup if we got a future
+                                    if let Some(close_future) = close_result {
+                                        if let Err(e) = close_future.await {
+                                            eprintln!("[SSE WARNING] Error during async generator cleanup on client disconnect: {}", e);
+                                        }
+                                    }
                                     exhausted = true;
                                     break;
                                 }
-                                send_count += 1;
-                                chunk_count += 1;
                             }
                         }
                         Err(e) => {
@@ -317,54 +266,89 @@ pub fn create_python_stream(
                 if got_stop_iteration {
                     exhausted = true;
                 }
-                if let Some(start) = convert_start {
-                    eprintln!(
-                        "[TIMING] Batch {} convert & send ({} chunks): {:?}",
-                        batch_count,
-                        send_count,
-                        start.elapsed()
-                    );
-                    let elapsed = start.elapsed();
-                    total_gil_time += elapsed;
-                    total_send_time += elapsed;
-                }
-                batch_count += 1;
             }
 
-            if let Some(start) = task_start {
-                let total = start.elapsed();
-                eprintln!("[TIMING] Async streaming complete (OPTIMIZED):");
-                eprintln!("  Total time: {:?}", total);
-                eprintln!("  Chunks sent: {}", chunk_count);
-                eprintln!("  Batches processed: {}", batch_count);
-                eprintln!("  Await time: {:?}", total_await_time);
-                eprintln!("  Send time: {:?}", total_send_time);
-                eprintln!("  GIL time: {:?}", total_gil_time);
+            // Ensure async generator cleanup runs
+            let close_result = Python::attach(|py| {
+                let iter_bound = async_iter.bind(py);
+                if iter_bound.hasattr("aclose").unwrap_or(false) {
+                    if let Ok(awaitable) = iter_bound.call_method0("aclose") {
+                        if let Some(locals) = TASK_LOCALS.get() {
+                            return pyo3_async_runtimes::into_future_with_locals(locals, awaitable).ok();
+                        } else {
+                            eprintln!("[SSE WARNING] Unable to get task locals for async generator cleanup at end of stream");
+                        }
+                    } else {
+                        eprintln!("[SSE WARNING] Failed to call aclose() on async generator at end of stream cleanup");
+                    }
+                }
+                None
+            });
+            if let Some(close_future) = close_result {
+                if let Err(e) = close_future.await {
+                    eprintln!("[SSE WARNING] Error during async generator cleanup at end of stream: {}", e);
+                }
+            }
+
+        });
+
+        // Async streaming successful, return stream
+        let s = stream::unfold(rx, |mut rx| async move {
+            match rx.recv().await {
+                Some(item) => Some((item, rx)),
+                None => None,
             }
         });
+        return Box::pin(s);
     } else {
-        let debug_sync = debug_timing;
         let sync_batch = sync_batch_size;
-        tokio::task::spawn_blocking(move || {
+
+        // OPTION 3: Use std::thread::spawn() instead of spawn_blocking()
+        // This avoids Tokio's blocking thread pool limit entirely
+        // Each sync SSE connection runs on its own dedicated OS thread
+
+        // Make tx cloneable for the spawn failure case
+        let tx_for_spawn = tx.clone();
+
+        // Check connection limits to prevent thread exhaustion DoS
+        let max_threads = get_max_sync_streaming_threads();
+        let current_threads = ACTIVE_SYNC_STREAMING_THREADS.load(Ordering::Relaxed);
+
+        if current_threads >= max_threads {
+            eprintln!("[SSE WARNING] Sync streaming thread limit reached: {} >= {}", current_threads, max_threads);
+            // Spawn async task to send retry directive (can't use blocking_send from runtime)
+            tokio::spawn({
+                let tx_clone = tx.clone();
+                async move {
+                    // RFC 6553 Server-Sent Events: send retry directive before closing
+                    let retry_directive = b"retry: 30000\n\n";
+                    let _ = tx_clone.send(Ok(Bytes::from_static(retry_directive))).await;
+                }
+            });
+            drop(tx);
+            let s = stream::unfold(rx, |mut rx| async move {
+                match rx.recv().await {
+                    Some(item) => Some((item, rx)),
+                    None => None,
+                }
+            });
+            return Box::pin(s);
+        }
+
+        // Increment active thread counter
+        ACTIVE_SYNC_STREAMING_THREADS.fetch_add(1, Ordering::Relaxed);
+
+        // Use Builder::new() to get a Result on thread spawn failure
+        match std::thread::Builder::new()
+            .name("sync-sse-generator".to_string())
+            .spawn(move || {
             let mut iterator: Option<Py<PyAny>> = None;
-            let mut chunk_count = 0u32;
-            let mut batch_count = 0u32;
-            let mut total_gil_time = std::time::Duration::ZERO;
-            let mut total_send_time = std::time::Duration::ZERO;
-            let task_start = if debug_sync {
-                Some(Instant::now())
-            } else {
-                None
-            };
             let mut batch_buffer = Vec::with_capacity(sync_batch);
+            let mut exhausted = false;
+
             loop {
-                let gil_start = if debug_sync {
-                    Some(Instant::now())
-                } else {
-                    None
-                };
                 batch_buffer.clear();
-                let exhausted = Python::attach(|py| {
+                let python_exhausted = Python::attach(|py| {
                     if iterator.is_none() {
                         let iter_target = resolved_target_final.clone_ref(py);
                         let bound = iter_target.bind(py);
@@ -399,48 +383,76 @@ pub fn create_python_stream(
                     }
                     false
                 });
-                if let Some(start) = gil_start {
-                    total_gil_time += start.elapsed();
+                if python_exhausted {
+                    exhausted = true;
                 }
                 if batch_buffer.is_empty() && exhausted {
                     break;
                 }
-                let send_start = if debug_sync {
-                    Some(Instant::now())
-                } else {
-                    None
-                };
                 for bytes in batch_buffer.drain(..) {
+                    // Use blocking_send which works from non-async context
                     if tx.blocking_send(Ok(bytes)).is_err() {
+                        // Client disconnected - close the generator to run cleanup code
+                        if let Some(ref iter) = iterator {
+                            Python::attach(|py| {
+                                match iter.bind(py).call_method0("close") {
+                                    Ok(_) => {},
+                                    Err(e) => {
+                                        eprintln!("[SSE WARNING] Error during sync generator cleanup on client disconnect: {}", e);
+                                    }
+                                }
+                            });
+                        }
+                        exhausted = true;
                         break;
                     }
-                    chunk_count += 1;
                 }
-                if let Some(start) = send_start {
-                    total_send_time += start.elapsed();
-                }
-                batch_count += 1;
                 if exhausted {
                     break;
                 }
             }
-            if let Some(start) = task_start {
-                let total = start.elapsed();
-                eprintln!("[TIMING] Sync streaming complete (OPTIMIZED):");
-                eprintln!("  Total time: {:?}", total);
-                eprintln!("  Chunks sent: {}", chunk_count);
-                eprintln!("  Batches processed: {}", batch_count);
-                eprintln!("  GIL time: {:?}", total_gil_time);
-                eprintln!("  Send time: {:?}", total_send_time);
+
+            // Ensure sync generator cleanup runs
+            if let Some(ref iter) = iterator {
+                Python::attach(|py| {
+                    match iter.bind(py).call_method0("close") {
+                        Ok(_) => {},
+                        Err(e) => {
+                            eprintln!("[SSE WARNING] Error during sync generator cleanup at end of stream: {}", e);
+                        }
+                    }
+                });
+            }
+            // Decrement thread counter when thread finishes
+            ACTIVE_SYNC_STREAMING_THREADS.fetch_sub(1, Ordering::Relaxed);
+        }) {
+            Ok(_) => {
+                // Thread spawned successfully, SSE will start streaming
+            }
+            Err(e) => {
+                eprintln!("[SSE ERROR] Failed to spawn sync streaming thread: {}", e);
+                // Decrement counter since thread spawn failed
+                ACTIVE_SYNC_STREAMING_THREADS.fetch_sub(1, Ordering::Relaxed);
+                // Spawn async task to send retry directive (can't use blocking_send from runtime)
+                tokio::spawn({
+                    let tx_clone = tx_for_spawn.clone();
+                    async move {
+                        // RFC 6553 Server-Sent Events: send retry directive before closing
+                        let retry_directive = b"retry: 30000\n\n";
+                        let _ = tx_clone.send(Ok(Bytes::from_static(retry_directive))).await;
+                    }
+                });
+                drop(tx_for_spawn);
+            }
+        }
+
+        // Create simple stream without error state in closure (keeps Stream trait bounds clean)
+        let s = stream::unfold(rx, |mut rx| async move {
+            match rx.recv().await {
+                Some(item) => Some((item, rx)),
+                None => None,
             }
         });
+        Box::pin(s)
     }
-
-    let s = stream::unfold(rx, |mut rx| async move {
-        match rx.recv().await {
-            Some(item) => Some((item, rx)),
-            None => None,
-        }
-    });
-    Box::pin(s)
 }
