@@ -16,7 +16,11 @@ use crate::router::{parse_query_string, Router};
 
 // Actix testing imports
 use actix_web::dev::Service;
-use actix_web::{test, web, App};
+use actix_web::http::header::{
+    ACCESS_CONTROL_ALLOW_CREDENTIALS, ACCESS_CONTROL_ALLOW_HEADERS, ACCESS_CONTROL_ALLOW_METHODS,
+    ACCESS_CONTROL_ALLOW_ORIGIN, ACCESS_CONTROL_EXPOSE_HEADERS, ACCESS_CONTROL_MAX_AGE, VARY,
+};
+use actix_web::{test, web, App, HttpResponse};
 use bytes::Bytes;
 
 // Macro for conditional debug output - only enabled with DJANGO_BOLT_TEST_DEBUG env var
@@ -320,19 +324,12 @@ pub fn handle_test_request_for(
         headers: header_map,
         cookies,
         context,
+        user: None,
     };
     let request_obj = Py::new(py, request)?;
 
-    // Call dispatch to get coroutine
-    let coroutine = dispatch.call1(py, (route, request_obj, handler_id))?;
-    test_debug!("[test_state] obtained coroutine");
-
-    // Instead of using pyo3_async_runtimes which requires a running event loop,
-    // let's run the coroutine directly using asyncio.run() or loop.run_until_complete()
+    // Get or create event loop upfront (needed for all handlers and streaming responses)
     let asyncio = py.import("asyncio")?;
-    test_debug!("[test_state] imported asyncio");
-
-    // Get or create event loop
     let loop_obj = if let Some(ev_obj) = event_loop_obj_opt {
         test_debug!("[test_state] using stored event loop");
         ev_obj.into_bound(py)
@@ -354,10 +351,14 @@ pub fn handle_test_request_for(
         }
     };
 
+    // All handlers (sync and async) go through async dispatch
+    // Sync handlers are executed in thread pool via sync_to_thread() in Python layer
+    test_debug!("[test_state] calling dispatch");
+    let coroutine = dispatch.call1(py, (route, request_obj, handler_id))?;
+    test_debug!("[test_state] obtained coroutine");
+
     test_debug!("[test_state] running coroutine with run_until_complete");
-    let result_obj = loop_obj.call_method1("run_until_complete", (coroutine,))?;
-    test_debug!("[test_state] coroutine completed");
-    let result_obj = result_obj.unbind();
+    let result_obj = loop_obj.call_method1("run_until_complete", (coroutine,))?.unbind();
 
     // Debug: check what type we got back
     let type_name = result_obj
@@ -495,7 +496,6 @@ pub fn handle_test_request_for(
         if has_aiter {
             // For async generators, we need to consume them with the event loop
             // Create a list from the async generator
-            let _asyncio = py.import("asyncio")?;
             let list_from_agen =
                 |agen: &Bound<PyAny>, loop_obj: &Bound<PyAny>| -> PyResult<Vec<Vec<u8>>> {
                     let loop_ref = loop_obj.clone();
@@ -691,13 +691,6 @@ pub fn handle_actix_http_request(
                 // Handle CORS preflight - MUST validate origin per RFC 6454
                 if is_preflight && !should_skip_cors {
                     if let Some(ref cors_cfg) = cors_config {
-                        use actix_web::http::header::{
-                            ACCESS_CONTROL_ALLOW_CREDENTIALS, ACCESS_CONTROL_ALLOW_HEADERS,
-                            ACCESS_CONTROL_ALLOW_METHODS, ACCESS_CONTROL_ALLOW_ORIGIN,
-                            ACCESS_CONTROL_MAX_AGE, VARY,
-                        };
-                        use actix_web::HttpResponse;
-
                         // Merge route-specific origins with global origins
                         let origins = if !cors_cfg.origins.is_empty() {
                             &cors_cfg.origins
@@ -819,15 +812,17 @@ pub fn handle_actix_http_request(
                 if let (Some(handler_id), Some(rate_cfg)) =
                     (handler_id_opt, rate_limit_config.as_ref())
                 {
-                    use crate::middleware::rate_limit::check_rate_limit;
                     // Convert headers to AHashMap
                     let header_map: ahash::AHashMap<String, String> = headers
                         .iter()
                         .map(|(k, v)| (k.to_lowercase(), v.clone()))
                         .collect();
-                    if let Some(response) =
-                        check_rate_limit(handler_id, &header_map, None, rate_cfg)
-                    {
+                    if let Some(response) = crate::middleware::rate_limit::check_rate_limit(
+                        handler_id,
+                        &header_map,
+                        None,
+                        rate_cfg,
+                    ) {
                         return Ok(response);
                     }
                 }
@@ -865,11 +860,6 @@ pub fn handle_actix_http_request(
                         // Add CORS headers to response if not skipped
                         if !should_skip_cors {
                             if let Some(ref cors_cfg) = cors_config {
-                                use actix_web::http::header::{
-                                    ACCESS_CONTROL_ALLOW_CREDENTIALS, ACCESS_CONTROL_ALLOW_ORIGIN,
-                                    ACCESS_CONTROL_EXPOSE_HEADERS, VARY,
-                                };
-
                                 // Merge route-specific origins with global origins
                                 let origins = if !cors_cfg.origins.is_empty() {
                                     &cors_cfg.origins
